"use strict";

require("./mocha-config.js");

var Promise = require("bluebird");

var HttpStatus = require('http-status-codes');

var Alias = require('../../main/js/map/data/Alias');
var Configuration = require('../../main/js/Configuration');
var LayoutAlias = require('../../main/js/map/data/LayoutAlias');
var MapModel = require('../../main/js/map/data/MapModel');
var NetworkError = require('../../main/js/NetworkError');
var Project = require('../../main/js/map/data/Project');
var Reaction = require('../../main/js/map/data/Reaction');
var ServerConnector = require('../../main/js/ServerConnector');

var logger = require('./logger');
var chai = require('chai');
var assert = chai.assert;

describe('ServerConnector', function() {
  describe('getProject', function() {
    it('default', function() {
      return ServerConnector.getProject().then(function(result) {
        assert.ok(result instanceof Project);
        assert.equal(result.getProjectId(), "sample");
        assert.equal(logger.getWarnings().length, 0);
      });
    });
    it('invalid project id', function() {
      return ServerConnector.getProject("invalid_project_id").then(function(result) {
        assert.equal(result, null);
      });
    });
  });

  it('getModels', function() {
    return ServerConnector.getModels("sample").then(function(models) {
      assert.equal(1, models.length);
      assert.ok(models[0] instanceof MapModel);
    });
  });

  it('getPublications', function() {
    return ServerConnector.getPublications().then(function(result) {
      assert.equal(result.totalSize, 1);
    });
  });

  it('getProjectId from GET params', function() {
    GuiConnector.getParams['id'] = "test";
    return ServerConnector.getProjectId().then(function(result) {
      assert.equal(result, "test");
    });
  });

  it('getReactions with empty list of ids', function() {
    return ServerConnector.getReactions([]).then(function(result) {
      assert.equal(result.length, 2);
      var reaction = result[0];
      assert.ok(reaction instanceof Reaction);
      assert.equal(reaction.getId(), 153513);
      assert.equal(reaction.getModelId(), 15781);
    });
  });

  it('getReactions without ids', function() {
    return ServerConnector.getReactions([]).then(function(result) {
      assert.equal(result.length, 2);
      var reaction = result[0];
      assert.ok(reaction instanceof Reaction);
      assert.equal(reaction.getId(), 153513);
      assert.equal(reaction.getModelId(), 15781);
    });
  });

  it('getElements with empty list of ids', function() {
    return ServerConnector.getAliases({}).then(function(result) {
      assert.equal(result.length, 30);
      var alias = result[0];
      assert.ok(alias instanceof Alias);
      assert.equal(alias.getModelId(), 15781);
    });
  });

  it('getOverlayElements', function() {
    return ServerConnector.getOverlayElements(18077).then(function(result) {
      assert.equal(result.length, 1);
      var layoutAlias = result[0];
      assert.ok(layoutAlias instanceof LayoutAlias);
      assert.equal(-7602176, layoutAlias.getColor().value);
      assert.equal(15781, layoutAlias.getModelId());
      assert.equal(329163, layoutAlias.getId());
    });
  });

  it('idsToString', function() {
    var ids = [ 3, 2, 9, 1, 6, 8, 3, 2, 9, 1, 7, 3 ];
    var str = ServerConnector.idsToString(ids);
    assert.equal(str, "1,2,3,6,7,8,9");
  });

  it('getOverlaySourceDownloadUrl', function() {
    var id = 17296;
    return ServerConnector.getOverlaySourceDownloadUrl({
      overlayId : id
    }).then(function(url) {
      assert.ok(url);
      assert.ok(url.indexOf(id) >= 0);
      return ServerConnector.readFile(url);
    });
  });

  it('getImageDownloadUrl', function() {
    var modelId = 15781;
    return ServerConnector.getImageDownloadUrl({
      modelId : modelId,
      handlerClass : "lcsb.mapviewer.converter.graphics.PngImageGenerator",
    }).then(function(url) {
      assert.ok(url);
      assert.ok(url.indexOf(modelId) >= 0);
      return ServerConnector.readFile(url);
    });
  });

  it('getModelDownloadUrl', function() {
    var modelId = 15781;
    return ServerConnector.getModelDownloadUrl({
      modelId : modelId,
      handlerClass : "lcsb.mapviewer.converter.model.celldesigner.CellDesignerXmlParser",
    }).then(function(url) {
      assert.ok(url);
      assert.ok(url.indexOf(modelId) >= 0);
      return ServerConnector.readFile(url);
    });
  });

  it('getProjectSourceDownloadUrl', function() {
    return ServerConnector.getProjectSourceDownloadUrl().then(function(url) {
      assert.ok(url);
      return ServerConnector.readFile(url);
    });
  });

  it('addOverlay', function() {
    return ServerConnector.addOverlay({
      name : "test nam",
      description : "test desc",
      content : "name color\nCAPN1 #00FF00\nPARK7 #AC0000",
      filename : "test.txt"
    }).then(function(overlayId) {
      assert.ok(overlayId);
    });
  });

  it('removeOverlay', function() {
    return ServerConnector.removeOverlay({
      overlayId : 17296,
    });
  });

  it('removeComment', function() {
    return ServerConnector.removeComment({
      commentId : 4290,
    });
  });

  it('updateOverlay', function() {
    return ServerConnector.updateOverlay({
      overlayId : 17296,
      name : "test nam2",
      description : "test desc2",
    });
  });

  it('logout', function() {
    return ServerConnector.logout().then(function() {
      assert.equal(ServerConnector.getSessionData().getToken(), undefined);
    });
  });

  it('getModelDownloadUrl', function() {
    return ServerConnector.getModelDownloadUrl({
      backgroundOverlayId : "cv14081",
    }).then(function(url) {
      assert.ok(url);
    });
  });

  it('getOverlayById', function() {
    return ServerConnector.getOverlayById(18083, "complex_model_with_submaps").then(function(overlay) {
      assert.ok(overlay);
    });
  });

  it('getConfiguration', function() {
    return ServerConnector.getConfiguration().then(function(configuration) {
      assert.ok(configuration instanceof Configuration);
      assert.ok(configuration.getElementTypes().length > 0);
      assert.ok(configuration.getReactionTypes().length > 0);
    });
  });

    describe('login', function() {
        it('try invalid credentials', function() {
            var method = ServerConnector.sendPostRequest;
            ServerConnector.sendPostRequest = function() {
                return Promise.reject(new NetworkError("xxx", {
                    statusCode : HttpStatus.FORBIDDEN
                }));
            };
            return ServerConnector.login("blabla", "blablabla").then(function() {
                ServerConnector.sendPostRequest = method;
                assert.ok(false);
            }, function(error) {
                ServerConnector.sendPostRequest = method;
                assert.ok(error.message.indexOf("credentials") >= 0);
            });
        });
    });

    describe('getServerBaseUrl', function() {
        it('url with GET arg that looks similar to original url', function() {
            helper.setUrl("http://localhost:8080/minerva/login.xhtml?from=http://localhost:8080/minerva/?id=sample");
            var url = ServerConnector.getServerBaseUrl();
            assert.ok(url.indexOf("?")===-1);
        });
    });



});
